// ../common/src/ast.ts
import { walkIdentifiers } from "@vue/compiler-sfc";
function isCallOf(node, test) {
  return !!(node && node.type === "CallExpression" && node.callee.type === "Identifier" && (typeof test === "string" ? node.callee.name === test : test(node.callee.name)));
}
function checkInvalidScopeReference(node, method, setupBindings) {
  if (!node)
    return;
  walkIdentifiers(node, (id) => {
    if (setupBindings.includes(id.name))
      throw new SyntaxError(
        `\`${method}()\` in <script setup> cannot reference locally declared variables (${id.name}) because it will be hoisted outside of the setup() function.`
      );
  });
}

// ../common/src/constants.ts
var DEFINE_OPTIONS = "defineOptions";

// ../common/src/context.ts
import { MagicString } from "vue/compiler-sfc";

// ../common/src/vue.ts
import { compileScript, parse } from "@vue/compiler-sfc";
var parseSFC = (code, id) => {
  const { descriptor } = parse(code, {
    filename: id
  });
  let compiledScript;
  if (descriptor.script || descriptor.scriptSetup) {
    compiledScript = compileScript(descriptor, {
      id
    });
  }
  return {
    ...descriptor,
    scriptCompiled: compiledScript
  };
};
var addToScript = ({
  sfc: { script, scriptCompiled },
  s,
  scriptCode
}) => {
  if (scriptCode.prepend.length + scriptCode.append.length === 0) {
    return;
  }
  if (script) {
    if (scriptCode.prepend)
      s.appendRight(script.loc.start.offset, scriptCode.prepend);
    if (scriptCode.append)
      s.appendRight(script.loc.end.offset, scriptCode.append);
  } else {
    const lang = scriptCompiled == null ? void 0 : scriptCompiled.attrs.lang;
    const attrs = lang ? ` lang="${lang}"` : "";
    s.prependLeft(
      0,
      `<script${attrs}>${scriptCode.prepend}
${scriptCode.append}<\/script>
`
    );
  }
};

// ../common/src/context.ts
var initContext = (code, id) => {
  let s;
  let sfc;
  const ctx = {
    code,
    id,
    scriptCode: {
      prepend: "",
      append: ""
    },
    get s() {
      return s || (s = new MagicString(code));
    },
    set s(_s) {
      s = _s;
    },
    get sfc() {
      return sfc || (sfc = parseSFC(code, id));
    },
    set sfc(_sfc) {
      sfc = _sfc;
    }
  };
  return {
    ctx,
    getMagicString() {
      return s;
    },
    getSFC() {
      return sfc;
    }
  };
};
var finalizeContext = (ctx) => {
  addToScript(ctx);
};

// ../common/src/unplugin.ts
var getTransformResult = (s, id) => {
  if (s && s.original !== s.toString()) {
    return {
      code: s.toString(),
      get map() {
        return s.generateMap({
          source: id,
          includeContent: true
        });
      }
    };
  }
};

// src/core/utils.ts
var filterMarco = (stmts) => {
  return stmts.map((raw) => {
    let node = raw;
    if (raw.type === "ExpressionStatement")
      node = raw.expression;
    return isCallOf(node, DEFINE_OPTIONS) ? node : void 0;
  }).filter((node) => !!node);
};
var hasPropsOrEmits = (node) => node.properties.some(
  (prop) => (prop.type === "ObjectProperty" || prop.type === "ObjectMethod") && prop.key.type === "Identifier" && (prop.key.name === "props" || prop.key.name === "emits")
);

export {
  checkInvalidScopeReference,
  DEFINE_OPTIONS,
  parseSFC,
  initContext,
  finalizeContext,
  getTransformResult,
  filterMarco,
  hasPropsOrEmits
};
